# -*- coding:utf-8 -*-
from __future__ import annotations

from typing import TYPE_CHECKING, Callable, Generic, List, Optional, Type, TypeVar, Union

if TYPE_CHECKING:
    # pylint: disable=cyclic-import
    from protobuf3_models.models import Model

Input = TypeVar("Input")
Output = TypeVar("Output")
ValidatorType = Callable[[Input], None]


class Field(Generic[Input, Output]):

    __slots__ = ("_name", "_protobuf_name", "_required", "_validators")

    _name: Optional[str]
    _protobuf_name: Optional[str]
    _validators: List[ValidatorType]

    def __init__(
        self,
        *,
        protobuf_name: Optional[str] = None,
        required: bool = True,
        validators: Optional[List[ValidatorType]] = None,
    ):
        self._name = None
        self._protobuf_name = protobuf_name
        self._required = required
        self._validators = validators or []

    def __set_name__(self, owner: Type[Model], name: str) -> None:
        self._name = name

        if self._protobuf_name is None:
            self._protobuf_name = name

    def __get__(self, instance: Optional[Model], owner: Type[Model]) -> Union[Field[Input, Output], Output]:
        if instance is None:
            return self

        if instance.message is None:
            raise Exception("No message attached to model.")

        return self.get(instance)

    def __set__(self, instance: Model, value: Input) -> None:
        if instance.message is None:
            raise Exception("No message attached to model.")

        self.set(instance, value)

    def __delete__(self, instance: Model) -> None:
        if instance.message is None:
            raise Exception("No message attached to model.")

        self.clear(instance)

    @property
    def name(self) -> str:
        if self._name is None:
            raise Exception("Field is not attached to model class.")

        return self._name

    @property
    def protobuf_name(self) -> str:
        if self._protobuf_name is None:
            raise Exception("Field is not attached to model class.")

        return self._protobuf_name

    def get(self, model: Model) -> Output:
        raise NotImplementedError()

    def set(self, model: Model, value: Input) -> None:
        raise NotImplementedError()

    def clear(self, model: Model) -> None:
        raise NotImplementedError()


class Int(Field[int, int]):
    def get(self, model: Model) -> int:
        return int(getattr(model.message, self.protobuf_name))

    def set(self, model: Model, value: int) -> None:
        setattr(model.message, self.protobuf_name, value)

    def clear(self, model: Model) -> None:
        model.message.ClearField(self.protobuf_name)



# -*- coding:utf-8 -*-
import tests.protobuf3_models_tests_pb2 as pb2  # pylint: disable=unused-import
from protobuf3_models.add import add
from protobuf3_models.fields import Int
from protobuf3_models.models import Model


def test_dummy() -> None:
    assert 2 + 2 == 4


def test_add() -> None:
    assert add(2, 2) == 4


def test_model() -> None:
    class ModelOptional(Model, protobuf_message=pb2.Int64):
        value = Int()

    model = ModelOptional()
    model._message = pb2.Int64()  # pylint: disable=protected-access
    model.value = 9



# -*- coding:utf-8 -*-
from __future__ import annotations

from dataclasses import dataclass
from types import MappingProxyType
from typing import Any, Dict, Iterator, Optional, Tuple, Type, Union, cast

from google.protobuf.pyext._message import Message  # pylint: disable=no-name-in-module

from protobuf3_models.fields import Field


class _Ignore:
    # pylint: disable=too-few-public-methods
    __slots__: tuple = tuple()


@dataclass(frozen=True)
class ModelOptions:
    message_class: Type[Message]


def _filter_fields(attrs: Dict[str, Any]) -> Iterator[Tuple[str, Field]]:
    for name, attr in attrs.items():
        if isinstance(attr, Field):
            yield name, attr


class ModelMeta(type):
    def __new__(
        mcs: Type[ModelMeta],
        name: str,
        bases: Tuple[Type],
        attrs: Dict[str, Any],
        protobuf_message: Union[None, Type[_Ignore], Type[Message]] = None,
    ) -> ModelMeta:
        if protobuf_message is _Ignore:
            return cast(ModelMeta, super().__new__(mcs, name, bases, attrs))

        if protobuf_message is None:
            raise RuntimeError(f"protobuf_message argument of class {name} must be set")

        if not issubclass(protobuf_message, Message):
            raise RuntimeError("protobuf_message must be a subclass of Protobuf message")

        # TODO: Validate fields against protobuf message definition
        attrs["_model_options"] = ModelOptions(
            message_class=protobuf_message,
        )

        mcs._make_fields(attrs)
        mcs._make_slots(attrs)

        cls = super().__new__(mcs, name, bases, attrs)

        return cast(ModelMeta, cls)

    @staticmethod
    def _make_fields(attrs: Dict[str, Any]) -> None:
        fields = {}

        for attr_name, attr in _filter_fields(attrs):  # type: str, Field
            fields[attr_name] = attr

        attrs["_fields"] = MappingProxyType(fields)

    @staticmethod
    def _make_slots(attrs: Dict[str, Any]) -> None:
        attrs["__slots__"] = tuple()


class Model(metaclass=ModelMeta, protobuf_message=_Ignore):
    # pylint: disable=too-few-public-methods
    __slots__ = ("_message", "_fields", "_model_options")

    _message: Optional[Message]
    _fields: Dict[str, Field]
    _model_options: ModelOptions

    @property
    def message(self) -> Message:
        if self._message is None:
            raise Exception("No message attached to model.")

        return self._message

    @property
    def fields(self) -> Dict[str, Field]:
        return self._fields

    @property
    def model_options(self) -> ModelOptions:
        return self._model_options



# -*- coding:utf-8 -*-
import os

from setuptools import find_packages, setup

from setup_commands.clean import CleanCommand
from setup_commands.protoc import ProtocCommand

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


with open("README.md") as fh:
    README = fh.read()


def scm_factory():
    from setuptools_scm.version import get_no_local_node  # pylint: disable=import-outside-toplevel

    def clean_scheme(version):
        """ Do not add commit hash to version."""
        return get_no_local_node(version)

    def distance_dev_version(version):
        """Do not try to guess and display next version number.

        Only add distance from previous tag.
        """
        if version.exact:
            return version.format_with("{tag}")

        return version.format_with("{tag}.dev{distance}")

    return {
        "local_scheme": clean_scheme,
        "version_scheme": distance_dev_version,
    }


setup(
    name="protobuf3-models",
    use_scm_version=scm_factory,
    description="Straightforward protobuf 3 handling through Python classes.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/mlga/protobuf3-models",
    author="mlga",
    author_email="github@mlga.io",
    license="MIT",
    keywords="protobuf validation serialization",
    setup_requires=[
        "pytest-runner~=5.1",
        "setuptools_scm~=4.1",
    ],
    install_requires=[
        "protobuf~=3.9",
    ],
    tests_require=[
        "pytest~=6.1",
        "pytest-cov~=2.7",
        "pytest-html~=2.1",
    ],
    extras_require={
        "develop": [
            "isort~=5.6",
            "mypy==0.790",
            "pre-commit~=2.8",
            "pylint~=2.3",
            "pytest~=6.1",
            "pytest-cov~=2.7",
            "pytest-html~=2.1",
            "Sphinx==3.0.1",
            "sphinx-rtd-theme==0.4.3",
        ],
    },
    packages=find_packages(exclude=["tests*", "examples*"]),
    include_package_data=True,
    platforms="any",
    zip_safe=False,
    cmdclass={
        "clean": CleanCommand,
        "protoc": ProtocCommand,
    },
    python_requires=">=3.7",
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tests/protobuf3_models_tests.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="tests/protobuf3_models_tests.proto",
    package="schematics_proto3.tests",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=_b(
        '\n"tests/protobuf3_models_tests.proto\x12\x17schematics_proto3.tests\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto"e\n\x06Nested\x12\x34\n\x05inner\x18\x01 \x01(\x0b\x32%.schematics_proto3.tests.Nested.Inner\x12\r\n\x05other\x18\x02 \x01(\t\x1a\x16\n\x05Inner\x12\r\n\x05value\x18\x01 \x01(\t">\n\rWrappedDouble\x12-\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue"<\n\x0cWrappedFloat\x12,\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1b.google.protobuf.FloatValue"<\n\x0cWrappedInt64\x12,\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1b.google.protobuf.Int64Value">\n\rWrappedUInt64\x12-\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt64Value"<\n\x0cWrappedInt32\x12,\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1b.google.protobuf.Int32Value">\n\rWrappedUInt32\x12-\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value":\n\x0bWrappedBool\x12+\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue">\n\rWrappedString\x12-\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue"<\n\x0cWrappedBytes\x12,\n\x07wrapped\x18\x01 \x01(\x0b\x32\x1b.google.protobuf.BytesValue"6\n\tTimestamp\x12)\n\x05value\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp">\n\x11RepeatedTimestamp\x12)\n\x05value\x18\x01 \x03(\x0b\x32\x1a.google.protobuf.Timestamp"w\n\x0eOneOfTimestamp\x12.\n\x06value1\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValueH\x00\x12,\n\x06value2\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x42\x07\n\x05inner"\x17\n\x06\x44ouble\x12\r\n\x05value\x18\x01 \x01(\x01"\x16\n\x05\x46loat\x12\r\n\x05value\x18\x01 \x01(\x02"\x16\n\x05Int64\x12\r\n\x05value\x18\x01 \x01(\x03"\x17\n\x06UInt64\x12\r\n\x05value\x18\x01 \x01(\x04"\x16\n\x05Int32\x12\r\n\x05value\x18\x01 \x01(\x05"\x17\n\x06UInt32\x12\r\n\x05value\x18\x01 \x01(\r"\x15\n\x04\x42ool\x12\r\n\x05value\x18\x01 \x01(\x08"\x17\n\x06String\x12\r\n\x05value\x18\x01 \x01(\t"\x16\n\x05\x42ytes\x12\r\n\x05value\x18\x01 \x01(\x0c""\n\x11RepeatedPrimitive\x12\r\n\x05value\x18\x01 \x03(\t"f\n\x0eRepeatedNested\x12<\n\x05inner\x18\x01 \x03(\x0b\x32-.schematics_proto3.tests.RepeatedNested.Inner\x1a\x16\n\x05Inner\x12\r\n\x05value\x18\x01 \x01(\t"=\n\x0fRepeatedWrapped\x12*\n\x05value\x18\x01 \x03(\x0b\x32\x1b.google.protobuf.Int32Value"=\n\x0eOneOfPrimitive\x12\x10\n\x06value1\x18\x01 \x01(\x04H\x00\x12\x10\n\x06value2\x18\x02 \x01(\tH\x00\x42\x07\n\x05inner"\x9c\x01\n\x0bOneOfNested\x12<\n\x06value1\x18\x01 \x01(\x0b\x32*.schematics_proto3.tests.OneOfNested.InnerH\x00\x12.\n\x06value2\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValueH\x00\x1a\x16\n\x05Inner\x12\r\n\x05value\x18\x01 \x01(\tB\x07\n\x05inner":\n\nSimpleEnum\x12,\n\x05value\x18\x01 \x01(\x0e\x32\x1d.schematics_proto3.tests.Enum"<\n\x0cRepeatedEnum\x12,\n\x05value\x18\x01 \x03(\x0e\x32\x1d.schematics_proto3.tests.Enum"u\n\tOneOfEnum\x12.\n\x06value1\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValueH\x00\x12/\n\x06value2\x18\x02 \x01(\x0e\x32\x1d.schematics_proto3.tests.EnumH\x00\x42\x07\n\x05inner**\n\x04\x45num\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x46IRST\x10\x01\x12\n\n\x06SECOND\x10\x02\x62\x06proto3'
    ),
    dependencies=[
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,
    ],
)

_ENUM = _descriptor.EnumDescriptor(
    name="Enum",
    full_name="schematics_proto3.tests.Enum",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(name="UNKNOWN", index=0, number=0, serialized_options=None, type=None),
        _descriptor.EnumValueDescriptor(name="FIRST", index=1, number=1, serialized_options=None, type=None),
        _descriptor.EnumValueDescriptor(name="SECOND", index=2, number=2, serialized_options=None, type=None),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1921,
    serialized_end=1963,
)
_sym_db.RegisterEnumDescriptor(_ENUM)

Enum = enum_type_wrapper.EnumTypeWrapper(_ENUM)
UNKNOWN = 0
FIRST = 1
SECOND = 2


_NESTED_INNER = _descriptor.Descriptor(
    name="Inner",
    full_name="schematics_proto3.tests.Nested.Inner",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Nested.Inner.value",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=207,
    serialized_end=229,
)

_NESTED = _descriptor.Descriptor(
    name="Nested",
    full_name="schematics_proto3.tests.Nested",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="inner",
            full_name="schematics_proto3.tests.Nested.inner",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="other",
            full_name="schematics_proto3.tests.Nested.other",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _NESTED_INNER,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=128,
    serialized_end=229,
)


_WRAPPEDDOUBLE = _descriptor.Descriptor(
    name="WrappedDouble",
    full_name="schematics_proto3.tests.WrappedDouble",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedDouble.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=231,
    serialized_end=293,
)


_WRAPPEDFLOAT = _descriptor.Descriptor(
    name="WrappedFloat",
    full_name="schematics_proto3.tests.WrappedFloat",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedFloat.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=295,
    serialized_end=355,
)


_WRAPPEDINT64 = _descriptor.Descriptor(
    name="WrappedInt64",
    full_name="schematics_proto3.tests.WrappedInt64",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedInt64.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=357,
    serialized_end=417,
)


_WRAPPEDUINT64 = _descriptor.Descriptor(
    name="WrappedUInt64",
    full_name="schematics_proto3.tests.WrappedUInt64",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedUInt64.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=419,
    serialized_end=481,
)


_WRAPPEDINT32 = _descriptor.Descriptor(
    name="WrappedInt32",
    full_name="schematics_proto3.tests.WrappedInt32",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedInt32.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=483,
    serialized_end=543,
)


_WRAPPEDUINT32 = _descriptor.Descriptor(
    name="WrappedUInt32",
    full_name="schematics_proto3.tests.WrappedUInt32",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedUInt32.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=545,
    serialized_end=607,
)


_WRAPPEDBOOL = _descriptor.Descriptor(
    name="WrappedBool",
    full_name="schematics_proto3.tests.WrappedBool",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedBool.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=609,
    serialized_end=667,
)


_WRAPPEDSTRING = _descriptor.Descriptor(
    name="WrappedString",
    full_name="schematics_proto3.tests.WrappedString",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedString.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=669,
    serialized_end=731,
)


_WRAPPEDBYTES = _descriptor.Descriptor(
    name="WrappedBytes",
    full_name="schematics_proto3.tests.WrappedBytes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="wrapped",
            full_name="schematics_proto3.tests.WrappedBytes.wrapped",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=733,
    serialized_end=793,
)


_TIMESTAMP = _descriptor.Descriptor(
    name="Timestamp",
    full_name="schematics_proto3.tests.Timestamp",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Timestamp.value",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=795,
    serialized_end=849,
)


_REPEATEDTIMESTAMP = _descriptor.Descriptor(
    name="RepeatedTimestamp",
    full_name="schematics_proto3.tests.RepeatedTimestamp",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.RepeatedTimestamp.value",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=851,
    serialized_end=913,
)


_ONEOFTIMESTAMP = _descriptor.Descriptor(
    name="OneOfTimestamp",
    full_name="schematics_proto3.tests.OneOfTimestamp",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value1",
            full_name="schematics_proto3.tests.OneOfTimestamp.value1",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value2",
            full_name="schematics_proto3.tests.OneOfTimestamp.value2",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="inner",
            full_name="schematics_proto3.tests.OneOfTimestamp.inner",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=915,
    serialized_end=1034,
)


_DOUBLE = _descriptor.Descriptor(
    name="Double",
    full_name="schematics_proto3.tests.Double",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Double.value",
            index=0,
            number=1,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1036,
    serialized_end=1059,
)


_FLOAT = _descriptor.Descriptor(
    name="Float",
    full_name="schematics_proto3.tests.Float",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Float.value",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1061,
    serialized_end=1083,
)


_INT64 = _descriptor.Descriptor(
    name="Int64",
    full_name="schematics_proto3.tests.Int64",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Int64.value",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1085,
    serialized_end=1107,
)


_UINT64 = _descriptor.Descriptor(
    name="UInt64",
    full_name="schematics_proto3.tests.UInt64",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.UInt64.value",
            index=0,
            number=1,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1109,
    serialized_end=1132,
)


_INT32 = _descriptor.Descriptor(
    name="Int32",
    full_name="schematics_proto3.tests.Int32",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Int32.value",
            index=0,
            number=1,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1134,
    serialized_end=1156,
)


_UINT32 = _descriptor.Descriptor(
    name="UInt32",
    full_name="schematics_proto3.tests.UInt32",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.UInt32.value",
            index=0,
            number=1,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1158,
    serialized_end=1181,
)


_BOOL = _descriptor.Descriptor(
    name="Bool",
    full_name="schematics_proto3.tests.Bool",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Bool.value",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1183,
    serialized_end=1204,
)


_STRING = _descriptor.Descriptor(
    name="String",
    full_name="schematics_proto3.tests.String",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.String.value",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1206,
    serialized_end=1229,
)


_BYTES = _descriptor.Descriptor(
    name="Bytes",
    full_name="schematics_proto3.tests.Bytes",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.Bytes.value",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1231,
    serialized_end=1253,
)


_REPEATEDPRIMITIVE = _descriptor.Descriptor(
    name="RepeatedPrimitive",
    full_name="schematics_proto3.tests.RepeatedPrimitive",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.RepeatedPrimitive.value",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1255,
    serialized_end=1289,
)


_REPEATEDNESTED_INNER = _descriptor.Descriptor(
    name="Inner",
    full_name="schematics_proto3.tests.RepeatedNested.Inner",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.RepeatedNested.Inner.value",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=207,
    serialized_end=229,
)

_REPEATEDNESTED = _descriptor.Descriptor(
    name="RepeatedNested",
    full_name="schematics_proto3.tests.RepeatedNested",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="inner",
            full_name="schematics_proto3.tests.RepeatedNested.inner",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _REPEATEDNESTED_INNER,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1291,
    serialized_end=1393,
)


_REPEATEDWRAPPED = _descriptor.Descriptor(
    name="RepeatedWrapped",
    full_name="schematics_proto3.tests.RepeatedWrapped",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.RepeatedWrapped.value",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1395,
    serialized_end=1456,
)


_ONEOFPRIMITIVE = _descriptor.Descriptor(
    name="OneOfPrimitive",
    full_name="schematics_proto3.tests.OneOfPrimitive",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value1",
            full_name="schematics_proto3.tests.OneOfPrimitive.value1",
            index=0,
            number=1,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value2",
            full_name="schematics_proto3.tests.OneOfPrimitive.value2",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="inner",
            full_name="schematics_proto3.tests.OneOfPrimitive.inner",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=1458,
    serialized_end=1519,
)


_ONEOFNESTED_INNER = _descriptor.Descriptor(
    name="Inner",
    full_name="schematics_proto3.tests.OneOfNested.Inner",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.OneOfNested.Inner.value",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=207,
    serialized_end=229,
)

_ONEOFNESTED = _descriptor.Descriptor(
    name="OneOfNested",
    full_name="schematics_proto3.tests.OneOfNested",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value1",
            full_name="schematics_proto3.tests.OneOfNested.value1",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value2",
            full_name="schematics_proto3.tests.OneOfNested.value2",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _ONEOFNESTED_INNER,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="inner",
            full_name="schematics_proto3.tests.OneOfNested.inner",
            index=0,
            containing_type=None,
            fields=[],
        ),
    ],
    serialized_start=1522,
    serialized_end=1678,
)


_SIMPLEENUM = _descriptor.Descriptor(
    name="SimpleEnum",
    full_name="schematics_proto3.tests.SimpleEnum",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.SimpleEnum.value",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1680,
    serialized_end=1738,
)


_REPEATEDENUM = _descriptor.Descriptor(
    name="RepeatedEnum",
    full_name="schematics_proto3.tests.RepeatedEnum",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value",
            full_name="schematics_proto3.tests.RepeatedEnum.value",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1740,
    serialized_end=1800,
)


_ONEOFENUM = _descriptor.Descriptor(
    name="OneOfEnum",
    full_name="schematics_proto3.tests.OneOfEnum",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="value1",
            full_name="schematics_proto3.tests.OneOfEnum.value1",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value2",
            full_name="schematics_proto3.tests.OneOfEnum.value2",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="inner", full_name="schematics_proto3.tests.OneOfEnum.inner", index=0, containing_type=None, fields=[]
        ),
    ],
    serialized_start=1802,
    serialized_end=1919,
)

_NESTED_INNER.containing_type = _NESTED
_NESTED.fields_by_name["inner"].message_type = _NESTED_INNER
_WRAPPEDDOUBLE.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._DOUBLEVALUE
_WRAPPEDFLOAT.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_WRAPPEDINT64.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._INT64VALUE
_WRAPPEDUINT64.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._UINT64VALUE
_WRAPPEDINT32.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._INT32VALUE
_WRAPPEDUINT32.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._UINT32VALUE
_WRAPPEDBOOL.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._BOOLVALUE
_WRAPPEDSTRING.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_WRAPPEDBYTES.fields_by_name["wrapped"].message_type = google_dot_protobuf_dot_wrappers__pb2._BYTESVALUE
_TIMESTAMP.fields_by_name["value"].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_REPEATEDTIMESTAMP.fields_by_name["value"].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ONEOFTIMESTAMP.fields_by_name["value1"].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_ONEOFTIMESTAMP.fields_by_name["value2"].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ONEOFTIMESTAMP.oneofs_by_name["inner"].fields.append(_ONEOFTIMESTAMP.fields_by_name["value1"])
_ONEOFTIMESTAMP.fields_by_name["value1"].containing_oneof = _ONEOFTIMESTAMP.oneofs_by_name["inner"]
_ONEOFTIMESTAMP.oneofs_by_name["inner"].fields.append(_ONEOFTIMESTAMP.fields_by_name["value2"])
_ONEOFTIMESTAMP.fields_by_name["value2"].containing_oneof = _ONEOFTIMESTAMP.oneofs_by_name["inner"]
_REPEATEDNESTED_INNER.containing_type = _REPEATEDNESTED
_REPEATEDNESTED.fields_by_name["inner"].message_type = _REPEATEDNESTED_INNER
_REPEATEDWRAPPED.fields_by_name["value"].message_type = google_dot_protobuf_dot_wrappers__pb2._INT32VALUE
_ONEOFPRIMITIVE.oneofs_by_name["inner"].fields.append(_ONEOFPRIMITIVE.fields_by_name["value1"])
_ONEOFPRIMITIVE.fields_by_name["value1"].containing_oneof = _ONEOFPRIMITIVE.oneofs_by_name["inner"]
_ONEOFPRIMITIVE.oneofs_by_name["inner"].fields.append(_ONEOFPRIMITIVE.fields_by_name["value2"])
_ONEOFPRIMITIVE.fields_by_name["value2"].containing_oneof = _ONEOFPRIMITIVE.oneofs_by_name["inner"]
_ONEOFNESTED_INNER.containing_type = _ONEOFNESTED
_ONEOFNESTED.fields_by_name["value1"].message_type = _ONEOFNESTED_INNER
_ONEOFNESTED.fields_by_name["value2"].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_ONEOFNESTED.oneofs_by_name["inner"].fields.append(_ONEOFNESTED.fields_by_name["value1"])
_ONEOFNESTED.fields_by_name["value1"].containing_oneof = _ONEOFNESTED.oneofs_by_name["inner"]
_ONEOFNESTED.oneofs_by_name["inner"].fields.append(_ONEOFNESTED.fields_by_name["value2"])
_ONEOFNESTED.fields_by_name["value2"].containing_oneof = _ONEOFNESTED.oneofs_by_name["inner"]
_SIMPLEENUM.fields_by_name["value"].enum_type = _ENUM
_REPEATEDENUM.fields_by_name["value"].enum_type = _ENUM
_ONEOFENUM.fields_by_name["value1"].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_ONEOFENUM.fields_by_name["value2"].enum_type = _ENUM
_ONEOFENUM.oneofs_by_name["inner"].fields.append(_ONEOFENUM.fields_by_name["value1"])
_ONEOFENUM.fields_by_name["value1"].containing_oneof = _ONEOFENUM.oneofs_by_name["inner"]
_ONEOFENUM.oneofs_by_name["inner"].fields.append(_ONEOFENUM.fields_by_name["value2"])
_ONEOFENUM.fields_by_name["value2"].containing_oneof = _ONEOFENUM.oneofs_by_name["inner"]
DESCRIPTOR.message_types_by_name["Nested"] = _NESTED
DESCRIPTOR.message_types_by_name["WrappedDouble"] = _WRAPPEDDOUBLE
DESCRIPTOR.message_types_by_name["WrappedFloat"] = _WRAPPEDFLOAT
DESCRIPTOR.message_types_by_name["WrappedInt64"] = _WRAPPEDINT64
DESCRIPTOR.message_types_by_name["WrappedUInt64"] = _WRAPPEDUINT64
DESCRIPTOR.message_types_by_name["WrappedInt32"] = _WRAPPEDINT32
DESCRIPTOR.message_types_by_name["WrappedUInt32"] = _WRAPPEDUINT32
DESCRIPTOR.message_types_by_name["WrappedBool"] = _WRAPPEDBOOL
DESCRIPTOR.message_types_by_name["WrappedString"] = _WRAPPEDSTRING
DESCRIPTOR.message_types_by_name["WrappedBytes"] = _WRAPPEDBYTES
DESCRIPTOR.message_types_by_name["Timestamp"] = _TIMESTAMP
DESCRIPTOR.message_types_by_name["RepeatedTimestamp"] = _REPEATEDTIMESTAMP
DESCRIPTOR.message_types_by_name["OneOfTimestamp"] = _ONEOFTIMESTAMP
DESCRIPTOR.message_types_by_name["Double"] = _DOUBLE
DESCRIPTOR.message_types_by_name["Float"] = _FLOAT
DESCRIPTOR.message_types_by_name["Int64"] = _INT64
DESCRIPTOR.message_types_by_name["UInt64"] = _UINT64
DESCRIPTOR.message_types_by_name["Int32"] = _INT32
DESCRIPTOR.message_types_by_name["UInt32"] = _UINT32
DESCRIPTOR.message_types_by_name["Bool"] = _BOOL
DESCRIPTOR.message_types_by_name["String"] = _STRING
DESCRIPTOR.message_types_by_name["Bytes"] = _BYTES
DESCRIPTOR.message_types_by_name["RepeatedPrimitive"] = _REPEATEDPRIMITIVE
DESCRIPTOR.message_types_by_name["RepeatedNested"] = _REPEATEDNESTED
DESCRIPTOR.message_types_by_name["RepeatedWrapped"] = _REPEATEDWRAPPED
DESCRIPTOR.message_types_by_name["OneOfPrimitive"] = _ONEOFPRIMITIVE
DESCRIPTOR.message_types_by_name["OneOfNested"] = _ONEOFNESTED
DESCRIPTOR.message_types_by_name["SimpleEnum"] = _SIMPLEENUM
DESCRIPTOR.message_types_by_name["RepeatedEnum"] = _REPEATEDENUM
DESCRIPTOR.message_types_by_name["OneOfEnum"] = _ONEOFENUM
DESCRIPTOR.enum_types_by_name["Enum"] = _ENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Nested = _reflection.GeneratedProtocolMessageType(
    "Nested",
    (_message.Message,),
    dict(
        Inner=_reflection.GeneratedProtocolMessageType(
            "Inner",
            (_message.Message,),
            dict(
                DESCRIPTOR=_NESTED_INNER,
                __module__="tests.protobuf3_models_tests_pb2"
                # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Nested.Inner)
            ),
        ),
        DESCRIPTOR=_NESTED,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Nested)
    ),
)
_sym_db.RegisterMessage(Nested)
_sym_db.RegisterMessage(Nested.Inner)

WrappedDouble = _reflection.GeneratedProtocolMessageType(
    "WrappedDouble",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDDOUBLE,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedDouble)
    ),
)
_sym_db.RegisterMessage(WrappedDouble)

WrappedFloat = _reflection.GeneratedProtocolMessageType(
    "WrappedFloat",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDFLOAT,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedFloat)
    ),
)
_sym_db.RegisterMessage(WrappedFloat)

WrappedInt64 = _reflection.GeneratedProtocolMessageType(
    "WrappedInt64",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDINT64,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedInt64)
    ),
)
_sym_db.RegisterMessage(WrappedInt64)

WrappedUInt64 = _reflection.GeneratedProtocolMessageType(
    "WrappedUInt64",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDUINT64,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedUInt64)
    ),
)
_sym_db.RegisterMessage(WrappedUInt64)

WrappedInt32 = _reflection.GeneratedProtocolMessageType(
    "WrappedInt32",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDINT32,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedInt32)
    ),
)
_sym_db.RegisterMessage(WrappedInt32)

WrappedUInt32 = _reflection.GeneratedProtocolMessageType(
    "WrappedUInt32",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDUINT32,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedUInt32)
    ),
)
_sym_db.RegisterMessage(WrappedUInt32)

WrappedBool = _reflection.GeneratedProtocolMessageType(
    "WrappedBool",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDBOOL,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedBool)
    ),
)
_sym_db.RegisterMessage(WrappedBool)

WrappedString = _reflection.GeneratedProtocolMessageType(
    "WrappedString",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDSTRING,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedString)
    ),
)
_sym_db.RegisterMessage(WrappedString)

WrappedBytes = _reflection.GeneratedProtocolMessageType(
    "WrappedBytes",
    (_message.Message,),
    dict(
        DESCRIPTOR=_WRAPPEDBYTES,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.WrappedBytes)
    ),
)
_sym_db.RegisterMessage(WrappedBytes)

Timestamp = _reflection.GeneratedProtocolMessageType(
    "Timestamp",
    (_message.Message,),
    dict(
        DESCRIPTOR=_TIMESTAMP,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Timestamp)
    ),
)
_sym_db.RegisterMessage(Timestamp)

RepeatedTimestamp = _reflection.GeneratedProtocolMessageType(
    "RepeatedTimestamp",
    (_message.Message,),
    dict(
        DESCRIPTOR=_REPEATEDTIMESTAMP,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedTimestamp)
    ),
)
_sym_db.RegisterMessage(RepeatedTimestamp)

OneOfTimestamp = _reflection.GeneratedProtocolMessageType(
    "OneOfTimestamp",
    (_message.Message,),
    dict(
        DESCRIPTOR=_ONEOFTIMESTAMP,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.OneOfTimestamp)
    ),
)
_sym_db.RegisterMessage(OneOfTimestamp)

Double = _reflection.GeneratedProtocolMessageType(
    "Double",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DOUBLE,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Double)
    ),
)
_sym_db.RegisterMessage(Double)

Float = _reflection.GeneratedProtocolMessageType(
    "Float",
    (_message.Message,),
    dict(
        DESCRIPTOR=_FLOAT,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Float)
    ),
)
_sym_db.RegisterMessage(Float)

Int64 = _reflection.GeneratedProtocolMessageType(
    "Int64",
    (_message.Message,),
    dict(
        DESCRIPTOR=_INT64,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Int64)
    ),
)
_sym_db.RegisterMessage(Int64)

UInt64 = _reflection.GeneratedProtocolMessageType(
    "UInt64",
    (_message.Message,),
    dict(
        DESCRIPTOR=_UINT64,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.UInt64)
    ),
)
_sym_db.RegisterMessage(UInt64)

Int32 = _reflection.GeneratedProtocolMessageType(
    "Int32",
    (_message.Message,),
    dict(
        DESCRIPTOR=_INT32,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Int32)
    ),
)
_sym_db.RegisterMessage(Int32)

UInt32 = _reflection.GeneratedProtocolMessageType(
    "UInt32",
    (_message.Message,),
    dict(
        DESCRIPTOR=_UINT32,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.UInt32)
    ),
)
_sym_db.RegisterMessage(UInt32)

Bool = _reflection.GeneratedProtocolMessageType(
    "Bool",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BOOL,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Bool)
    ),
)
_sym_db.RegisterMessage(Bool)

String = _reflection.GeneratedProtocolMessageType(
    "String",
    (_message.Message,),
    dict(
        DESCRIPTOR=_STRING,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.String)
    ),
)
_sym_db.RegisterMessage(String)

Bytes = _reflection.GeneratedProtocolMessageType(
    "Bytes",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BYTES,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.Bytes)
    ),
)
_sym_db.RegisterMessage(Bytes)

RepeatedPrimitive = _reflection.GeneratedProtocolMessageType(
    "RepeatedPrimitive",
    (_message.Message,),
    dict(
        DESCRIPTOR=_REPEATEDPRIMITIVE,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedPrimitive)
    ),
)
_sym_db.RegisterMessage(RepeatedPrimitive)

RepeatedNested = _reflection.GeneratedProtocolMessageType(
    "RepeatedNested",
    (_message.Message,),
    dict(
        Inner=_reflection.GeneratedProtocolMessageType(
            "Inner",
            (_message.Message,),
            dict(
                DESCRIPTOR=_REPEATEDNESTED_INNER,
                __module__="tests.protobuf3_models_tests_pb2"
                # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedNested.Inner)
            ),
        ),
        DESCRIPTOR=_REPEATEDNESTED,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedNested)
    ),
)
_sym_db.RegisterMessage(RepeatedNested)
_sym_db.RegisterMessage(RepeatedNested.Inner)

RepeatedWrapped = _reflection.GeneratedProtocolMessageType(
    "RepeatedWrapped",
    (_message.Message,),
    dict(
        DESCRIPTOR=_REPEATEDWRAPPED,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedWrapped)
    ),
)
_sym_db.RegisterMessage(RepeatedWrapped)

OneOfPrimitive = _reflection.GeneratedProtocolMessageType(
    "OneOfPrimitive",
    (_message.Message,),
    dict(
        DESCRIPTOR=_ONEOFPRIMITIVE,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.OneOfPrimitive)
    ),
)
_sym_db.RegisterMessage(OneOfPrimitive)

OneOfNested = _reflection.GeneratedProtocolMessageType(
    "OneOfNested",
    (_message.Message,),
    dict(
        Inner=_reflection.GeneratedProtocolMessageType(
            "Inner",
            (_message.Message,),
            dict(
                DESCRIPTOR=_ONEOFNESTED_INNER,
                __module__="tests.protobuf3_models_tests_pb2"
                # @@protoc_insertion_point(class_scope:schematics_proto3.tests.OneOfNested.Inner)
            ),
        ),
        DESCRIPTOR=_ONEOFNESTED,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.OneOfNested)
    ),
)
_sym_db.RegisterMessage(OneOfNested)
_sym_db.RegisterMessage(OneOfNested.Inner)

SimpleEnum = _reflection.GeneratedProtocolMessageType(
    "SimpleEnum",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SIMPLEENUM,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.SimpleEnum)
    ),
)
_sym_db.RegisterMessage(SimpleEnum)

RepeatedEnum = _reflection.GeneratedProtocolMessageType(
    "RepeatedEnum",
    (_message.Message,),
    dict(
        DESCRIPTOR=_REPEATEDENUM,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.RepeatedEnum)
    ),
)
_sym_db.RegisterMessage(RepeatedEnum)

OneOfEnum = _reflection.GeneratedProtocolMessageType(
    "OneOfEnum",
    (_message.Message,),
    dict(
        DESCRIPTOR=_ONEOFENUM,
        __module__="tests.protobuf3_models_tests_pb2"
        # @@protoc_insertion_point(class_scope:schematics_proto3.tests.OneOfEnum)
    ),
)
_sym_db.RegisterMessage(OneOfEnum)


# @@protoc_insertion_point(module_scope)



# -*- coding:utf-8 -*-
import distutils
import subprocess
import sys


class ProtocCommand(distutils.cmd.Command):
    """A custom command to run protoc generator."""

    description = "Compile protoc definitions."
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run command."""
        commands = (
            [
                "protoc",
                "-I",
                ".",
                "--python_out",
                ".",
                "tests/protobuf3_models_tests.proto",
            ],
        )

        for command in commands:
            self.announce(f'Running command: {" ".join(command)}', level=distutils.log.INFO)
            try:
                subprocess.check_call(command)
            except subprocess.CalledProcessError as ex:
                sys.exit(ex.returncode)



# -*- coding:utf-8 -*-
import distutils
import subprocess
import sys


class CleanCommand(distutils.cmd.Command):
    """A custom command to clean build / test artifacts."""

    description = "Clean artifacts (reports, builds...)."
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run command."""
        commands = (
            ["rm", "-Rf", ".pytest_cache"],
            ["rm", "-Rf", "htmlcov"],
            ["rm", "-Rf", "htmltest"],
            ["rm", "-f", ".coverage"],
            ["rm", "-f", "coverage.xml"],
            ["rm", "-f", "report.xml"],
            ["rm", "-Rf", "build"],
            ["rm", "-Rf", "dist"],
            ["rm", "-Rf", ".mypy_cache"],
        )

        for command in commands:
            self.announce(f'Running command: {" ".join(command)}', level=distutils.log.INFO)
            try:
                subprocess.check_call(command)
            except subprocess.CalledProcessError as ex:
                sys.exit(ex.returncode)
